import type { NextApiHandler, NextApiRequest, NextApiResponse } from 'next';
import { ApiErrorHandler } from './error-handler';
/**
 *  an utility type to shorten writing Record<string, T = unknown>
 */
export declare type TypedObject<T = unknown> = Record<string, T>;
/**
 *  a standard next.js api handler but with req.middleware available
 *  see [official doc](https://nextjs.org/docs/api-routes/introduction) for more details
 */
export declare type NextApiHandlerWithMiddleware<T = unknown, M extends TypedObject = TypedObject> = (req: NextApiRequest<M>, res: NextApiResponse) => T | Promise<T>;
/**
 *  all api response combines success response & error response
 */
export declare type ApiResponse<T = unknown> = SuccessApiResponse<T> | ErrorApiResponse;
/**
 *  api response that has http status code 2xx with data
 */
export declare type SuccessApiResponse<T> = {
    success: true;
    data: T;
};
/**
 *  api response that has http status code 4xx or 5xx with error message
 */
export declare type ErrorApiResponse = {
    success: false;
    message: string;
};
/**
 * RouterBuilder options
 */
export declare type RouterOptions = Partial<{
    error: ApiErrorHandler;
    shoeMessage: boolean;
}>;
/**
 * A router builder is next api handler builder that expose express-like api.
 *
 * ### Example (basic usage)
 * ```js
 * // In next.js /pages/api/[route]
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().build()
 * export default handler
 * ```
 *
 * ### Example (add RESTful method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get((req, res) => "RESPONSE")
 * export default handler
 * ```
 *
 * ### Example (add async method)
 * ```js
 * import { RouterBuilder } from 'next-api-handler'
 * const handler = new RouterBuilder().get(async (req, res) => "ASYNC RESPONSE")
 * export default handler
 * ```
 *
 * @returns a builder that can build a next.js api handler.
 */
export declare class RouterBuilder {
    private readonly route;
    private readonly middlewareList;
    private readonly middlewareQueue;
    private readonly routerOptions;
    constructor(options?: RouterOptions);
    get<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    head<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    patch<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    options<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    connect<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    delete<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    trace<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    post<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    put<T, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    use<T extends TypedObject, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    inject<T extends TypedObject, M extends TypedObject = TypedObject>(handler: NextApiHandlerWithMiddleware<T, M>): RouterBuilder;
    build(): NextApiHandler;
    private handleMiddlewareQueue;
    private handleMiddlewareList;
    private add;
}
